#import "PendoFlutterPlugin.h"
#import <Pendo/Pendo.h>
@implementation PendoFlutterPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel
                                     methodChannelWithName:@"pendo_flutter_plugin"
                                     binaryMessenger:[registrar messenger]];
    PendoFlutterPlugin* instance = [[PendoFlutterPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    
    if([@"initPendoSDKWithParamaters" isEqualToString:call.method]) {
        
        if (nil != call.arguments && [call.arguments isKindOfClass: [NSDictionary class]]) {
            
            NSDictionary *arguments = (NSDictionary *)call.arguments;
            NSString     *appKey     = [arguments objectForKey:@"ApplicationKey"];
            NSDictionary *initParams = [arguments objectForKey:@"ApplicationParamaters"];
            [self initSDK:appKey initParams:initParams];
        }
        
    } else if([@"clearVisitor" isEqualToString:call.method]) {
        
        [self clearVisitor];
        
    } else if([@"switchVisitorByParamaters" isEqualToString:call.method]) {
            
        if (nil != call.arguments && [call.arguments isKindOfClass: [NSDictionary class]]) {
            NSDictionary *arguments = (NSDictionary *)call.arguments;
            NSString     *visitorId   = [arguments objectForKey:@"VisitorIDKey"];
            NSString     *accountId   = [arguments objectForKey:@"AccountIDKey"];
            NSDictionary *visitorData = [arguments objectForKey:@"VisitorDataKey"];
            NSDictionary *accountData = [arguments objectForKey:@"AccountDataKey"];
            [self switchVisitor:visitorId accountId:accountId visitorData:visitorData accountData:accountData];
        }
        
    } else if([@"setVisitorData" isEqualToString:call.method]) {
            
        if (nil != call.arguments && [call.arguments isKindOfClass: [NSDictionary class]]) {
            NSDictionary *arguments   = (NSDictionary *)call.arguments;
            NSDictionary *visitorData = [arguments objectForKey:@"VisitorDataKey"];
            [self setVisitorData:visitorData];
        }
        
    } else if([@"setAccountData" isEqualToString:call.method]) {
        
        if (nil != call.arguments && [call.arguments isKindOfClass: [NSDictionary class]]) {
            NSDictionary *arguments   = (NSDictionary *)call.arguments;
            NSDictionary *accountData = [arguments objectForKey:@"AccountDataKey"];
            [self setAccountData:accountData];
        }
        
    } else if([@"trackEventWithProperties" isEqualToString:call.method]) {
        
        if (nil != call.arguments && [call.arguments isKindOfClass: [NSDictionary class]]) {
            NSDictionary *arguments = (NSDictionary *)call.arguments;
            NSString     *eventName  = [arguments objectForKey:@"EventNameKey"];
            NSDictionary *properties = [arguments objectForKey:@"EventPropertiesKey"];
            [self track:eventName properties:properties];
        }
    } else {
        result(FlutterMethodNotImplemented);
    }
}

#pragma mark - SDK methods

- (void)initSDK:(nonnull NSString *)appKey initParams:(nullable NSDictionary *)initParams
{
    PendoInitParams *sdkInitParams = [[PendoInitParams alloc] init];
    if(initParams != nil) {
        
        if ([[initParams allKeys] containsObject:@"visitorId"]) {
            NSString *visitorId = initParams[@"visitorId"];
            [sdkInitParams setVisitorId:visitorId];
        }
        
        if ([[initParams allKeys] containsObject:@"accountId"]) {
            NSString *accountId = initParams[@"accountId"];
            [sdkInitParams setAccountId:accountId];
        }
        
        if ([[initParams allKeys] containsObject:@"visitorData"]) {
            
            NSDictionary *visitorData = initParams[@"visitorData"];
            [sdkInitParams setVisitorData:visitorData];
        }
        
        if ([[initParams allKeys] containsObject:@"accountData"]) {
            NSDictionary *accountData = initParams[@"accountData"];
            [sdkInitParams setAccountData:accountData];
        }    
    }
    
    NSDictionary *pendoOptions = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:YES] forKey:@"disableAnalytics"];
    [sdkInitParams setPendoOptions:pendoOptions];
    [[PendoManager sharedManager] initSDK:appKey initParams:sdkInitParams];
}
/**
 * Must be called <b>after</b> the SDK was initialized.
 * @brief Clears the current visitor.
 */
//  - (void)clearVisitor;
- (void)clearVisitor
{
    [[PendoManager sharedManager] clearVisitor];
}

/**
 * Must be called <b>after</b> the SDK was initialized.
 *
 * @brief Switch to a new visitor.
 *
 * @param visitorId The visitor's ID.
 * @param accountId The account's ID.
 * @param visitorData The visitor's data.
 * @param accountData The account's data.
 */
- (void)switchVisitor:(nullable NSString *)visitorId
            accountId:(nullable NSString *)accountId
          visitorData:(nullable NSDictionary *)visitorData
          accountData:(nullable NSDictionary *)accountData
{
    [[PendoManager sharedManager] switchVisitor:visitorId accountId:accountId visitorData:visitorData accountData:accountData];
}

/**
 * Set a visitor data value for a given data name.
 * This data is used by Pendo Mobile for creating audiences or reporting analytics.
 * For instance you might want to provide data on the visitor's age or if the visitor is logged into a service.
 * @code
 * [[PendoManager sharedManager] setVisitorData:@(YES) forKey:@"LoggedIn"]
 * @endcode
 * You can provide multiple visitor data to the initSDK method call or supply a dictionary of data.
 * @code
 * [PendoManager sharedManager].visitorData = @{@"key1" :@"value1", ...};
 * @endcode
 *
 */
- (void)setVisitorData:(nonnull NSDictionary *)visitorData
{
    for(id key in visitorData) {
        [[PendoManager sharedManager] setVisitorDataValue:[visitorData objectForKey:key] forKey:key];
    }
}

/**
 * Set a account data value for a given data name.
 * This data is used by Pendo Mobile for creating audiences or reporting analytics.
 * For instance you might want to provide data on the account's subscription or if the account is active or not.
 * @code
 * [[PendoManager sharedManager] setAccountData:@(YES) forKey:@"isPro"]
 * @endcode
 * You can provide multiple account data to the initSDK method call or supply a dictionary of data.
 * @code
 * [PendoManager sharedManager].accountData = @{@"key1" :@"value1", ...};
 * @endcode
 */
- (void)setAccountData:(nonnull NSDictionary *)accountData
{
    for(id key in accountData) {
        [[PendoManager sharedManager] setAccountDataValue:[accountData objectForKey:key] forKey:key];
    }
}

/**
 * When your application needs to send additional events about actions your users perform.
 *
 * @param event The event name describing the user’s action.
 * @param properties dictionary of event properties (optional).
 * @brief Queue a track eventsfor transmission, optionally including properties as the payload of the event.
 */
- (void)track:(nonnull NSString *)event properties:(nullable NSDictionary *)properties
{
    [[PendoManager sharedManager] track:event properties:properties];
}
@end
